package org.jboss.playground;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.ClientRequestContext;
import javax.ws.rs.client.ClientRequestFilter;
import javax.ws.rs.client.ClientResponseContext;
import javax.ws.rs.client.ClientResponseFilter;
import javax.ws.rs.client.Entity;
import javax.ws.rs.core.Cookie;
import javax.ws.rs.core.Form;
import javax.ws.rs.core.Response;
import javax.ws.rs.ext.Provider;

@Provider
public final class Authenticator implements ClientRequestFilter, ClientResponseFilter {

    private static final String COOKIE = "Cookie";

    // security params
    private static final String J_SECURITY_CHECK = "j_security_check";
    private static final String J_USERNAME = "j_username";
    private static final String J_PASSWORD = "j_password";

    private final String username;
    private final String password;
    private final String baseUri;

    public Authenticator() {
        this.username = null;
        this.password = null;
        this.baseUri = null;
    }

    public Authenticator(final String baseUri, final String username, final String password) {
        this.username = username;
        this.password = password;
        this.baseUri = baseUri;
    }

    // http://stackoverflow.com/questions/5536266/request-to-j-security-check-return-408-error-only-with-right-paramters
    /**
     * REQUEST
     */
    @Override
    public void filter(final ClientRequestContext requestContext) throws IOException {
        final List<Cookie> cookies = new ArrayList<>();

        final Client initialClient = ClientBuilder.newClient();
        final Response responseInitial = initialClient
                .target(requestContext.getUri())
                .request(requestContext.getAcceptableMediaTypes().get(0))
                .method(requestContext.getMethod());
        initialClient.close();
        responseInitial.getCookies().values().stream().forEach((cookie) -> {
            cookies.add(cookie);
        });

        final Client loginClient = ClientBuilder.newClient();
        final Form form = new Form();
        form.param(J_USERNAME, this.username);
        form.param(J_PASSWORD, this.password);
        final Response loginResponse = loginClient.target(this.baseUri).path(J_SECURITY_CHECK)
                .request()
                .cookie(cookies.get(0))
                .post(Entity.form(form));
        loginClient.close();
        loginResponse.getCookies().values().stream().forEach((cookie) -> {
            cookies.add(cookie);
        });

        final List<Object> cookieArray = new ArrayList<>();
        cookies.stream().forEach(cookie -> {
            cookieArray.add(cookie);
        });

        requestContext.getHeaders().put(COOKIE, cookieArray);
    }

    /**
     * RESPONSE
     */
    @Override
    public void filter(final ClientRequestContext crc, final ClientResponseContext crc1) throws IOException {
        final Client logoutClient = ClientBuilder.newClient();
        logoutClient.target(this.baseUri)
                .path("logout.jsp")
                .request()
                .cookie(crc1.getCookies().entrySet().stream().findFirst().get().getValue())
                .get();
        logoutClient.close();
    }
}
