package org.jboss.playground;

import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.batch.operations.JobSecurityException;
import javax.batch.operations.JobStartException;
import javax.batch.runtime.BatchRuntime;
import javax.batch.runtime.BatchStatus;
import javax.ejb.LocalBean;
import javax.ejb.Singleton;
import javax.json.JsonArray;
import javax.json.JsonObject;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

@Singleton
@LocalBean
@Path(value = "batch")
public class BatchManager {

    @Path(value = "start")
    @GET
    @Produces(value = "application/json")
    public long start(@QueryParam("job") String job) {
        try {
            return BatchRuntime.getJobOperator().start(job, new Properties());
        } catch (JobStartException | JobSecurityException e) {
            Logger.getLogger(getClass().getName()).log(
                    Level.SEVERE, e.getMessage(), e);
            return -1l;
        }
    }

    @Path(value = "status")
    @GET
    @Produces(value = MediaType.APPLICATION_JSON)
    public BatchStatus getStatus(@QueryParam("executionId") long executionId) {
        return BatchRuntime.getJobOperator().getJobExecution(executionId).getBatchStatus();
    }

    @Path(value = "test")
    @Produces(value = MediaType.APPLICATION_JSON)
    @GET
    public Response test() {
        final Client client = ClientBuilder.newClient().register(new Authenticator("http://base.url.goes.here", "test", "test"));
        final WebTarget target = client.target("http://base.url.goes.here/resources/person");
        final JsonArray response = target.request(MediaType.APPLICATION_JSON).get(JsonArray.class);
        System.out.println(response);
        client.close();
        return Response.ok().build();
    }
}
